@@ -118,14 +118,17 @@ static void unlock_mutex(pthread_mutex_t *l)
 		log_exit("%s - returned %d\n", strerror(ret), ret);
 }
 
-static void store_lock(void)
+static inline void unlock_mutex_function(pthread_mutex_t **mutex)
+{
+	if (*mutex)
+		unlock_mutex(*mutex);
+}
+#define __do_unlock call_cleaner(unlock_mutex)
+
+static pthread_mutex_t* __attribute__((warn_unused_result)) store_lock(void)
 {
 	lock_mutex(&pidns_store_mutex);
-}
-
-static void store_unlock(void)
-{
-	unlock_mutex(&pidns_store_mutex);
+	return &pidns_store_mutex;
 }
 
 /* /proc/       =    6
@@ -252,7 +255,7 @@ static void prune_initpid_store(void)
 }
 
 /* Must be called under store_lock */
-static void save_initpid(struct stat *sb, pid_t pid)
+static void save_initpid(ino_t pidns_inode, pid_t pid)
 {
 	__do_free struct pidns_init_store *entry = NULL;
 	__do_close int pidfd = -EBADF;
@@ -271,13 +274,13 @@ static void save_initpid(struct stat *sb, pid_t pid)
 	if (stat(path, &st))
 		return;
 
-	entry = malloc(sizeof(*entry));
-	if (entry)
+	entry = zalloc(sizeof(*entry));
+	if (!entry)
 		return;
 
 	ino_hash = HASH(entry->ino);
 	*entry = (struct pidns_init_store){
-		.ino		= sb->st_ino,
+		.ino		= pidns_inode,
 		.initpid	= pid,
 		.ctime		= st.st_ctime,
 		.next		= pidns_hash_table[ino_hash],
@@ -296,12 +299,12 @@ static void save_initpid(struct stat *sb, pid_t pid)
  * otherwise.
  * Must be called under store_lock
  */
-static struct pidns_init_store *lookup_verify_initpid(struct stat *sb)
+static struct pidns_init_store *lookup_verify_initpid(ino_t pidns_inode)
 {
-	struct pidns_init_store *entry = pidns_hash_table[HASH(sb->st_ino)];
+	struct pidns_init_store *entry = pidns_hash_table[HASH(pidns_inode)];
 
 	while (entry) {
-		if (entry->ino == sb->st_ino) {
+		if (entry->ino == pidns_inode) {
 			if (initpid_still_valid(entry)) {
 				entry->lastcheck = time(NULL);
 				return entry;
@@ -334,7 +337,7 @@ static int send_creds_clone_wrapper(void *arg)
  * stack sizes: 8MB.
  */
 #define __LXCFS_STACK_SIZE (8 * 1024 * 1024)
-static pid_t lxcfs_clone(int (*fn)(void *), void *arg, int flags)
+pid_t lxcfs_clone(int (*fn)(void *), void *arg, int flags)
 {
 	pid_t ret;
 	void *stack;
@@ -395,9 +398,13 @@ static pid_t get_init_pid_for_task(pid_t task)
 {
 	char v = '0';
 	pid_t pid_ret = -1;
+	struct ucred cred = {
+		.pid = -1,
+		.uid = -1,
+		.gid = -1,
+	};
 	pid_t pid;
 	int sock[2];
-	struct ucred cred;
 
 	if (socketpair(AF_UNIX, SOCK_DGRAM, 0, sock) < 0)
 		return -1;
@@ -428,6 +435,7 @@ out:
 
 pid_t lookup_initpid_in_store(pid_t pid)
 {
+	__do_unlock pthread_mutex_t *store_mutex = NULL;
 	pid_t answer = 0;
 	char path[LXCFS_PROC_PID_NS_LEN];
 	struct stat st;
@@ -435,19 +443,24 @@ pid_t lookup_initpid_in_store(pid_t pid)
 
 	snprintf(path, sizeof(path), "/proc/%d/ns/pid", pid);
 
-	store_lock();
 	if (stat(path, &st))
 		goto out;
 
-	entry = lookup_verify_initpid(&st);
+	store_mutex = store_lock();
+
+	entry = lookup_verify_initpid(st.st_ino);
 	if (entry) {
 		answer = entry->initpid;
 		goto out;
 	}
 
+	/* release the mutex as the following call is expensive */
+	unlock_mutex(move_ptr(store_mutex));
 	answer = get_init_pid_for_task(pid);
+	store_mutex = store_lock();
+
 	if (answer > 0)
-		save_initpid(&st, answer);
+		save_initpid(st.st_ino, answer);
 
 out:
 	/*
@@ -456,8 +469,6 @@ out:
 	 */
 	prune_initpid_store();
 
-	store_unlock();
-
 	return answer;
 }
 
