@@ -1019,7 +1019,7 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 	struct lxcfs_opts *opts = (struct lxcfs_opts *)fuse_get_context()->private_data;
 	struct file_info *d = INTTYPE_TO_PTR(fi->fh);
 	uint64_t memlimit = 0, memusage = 0, memswlimit = 0, memswusage = 0,
-		 hosttotal = 0;
+		 hosttotal = 0, swtotal = 0;
 	struct memory_stat mstat = {};
 	size_t linelen = 0, total_len = 0;
 	char *cache = d->buf;
@@ -1075,6 +1075,7 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 		if (safe_uint64(memswusage_str, &memswusage, 10) < 0)
 			lxcfs_error("Failed to convert memswusage %s", memswusage_str);
 		memswusage = memswusage / 1024;
+		swtotal = memswlimit;
 	}
 
 	if (safe_uint64(memusage_str, &memusage, 10) < 0)
@@ -1093,6 +1094,9 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 		memset(lbuf, 0, 100);
 		if (startswith(line, "MemTotal:")) {
 			sscanf(line+sizeof("MemTotal:")-1, "%" PRIu64, &hosttotal);
+			if (memlimit == 0)
+				memlimit = hosttotal;
+
 			if (hosttotal < memlimit)
 				memlimit = hosttotal;
 			snprintf(lbuf, 100, "MemTotal:       %8" PRIu64 " kB\n", memlimit);
@@ -1107,27 +1111,25 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 			uint64_t hostswtotal = 0;
 
 			sscanf(line + STRLITERALLEN("SwapTotal:"), "%" PRIu64, &hostswtotal);
-			if (hostswtotal < memswlimit)
-				memswlimit = hostswtotal;
 
-			if (memswlimit >= memlimit)
-				memswlimit -= memlimit;
+			/* Don't advertise more SWAP than the total memory allowed. */
+			if (hostswtotal < swtotal)
+				swtotal = hostswtotal;
 
-			snprintf(lbuf, 100, "SwapTotal:      %8" PRIu64 " kB\n", memswlimit);
+			snprintf(lbuf, 100, "SwapTotal:      %8" PRIu64 " kB\n", swtotal);
 			printme = lbuf;
 		} else if (startswith(line, "SwapTotal:") && opts && opts->swap_off == true) {
 			snprintf(lbuf, 100, "SwapTotal:      %8" PRIu64 " kB\n", (uint64_t)0);
 			printme = lbuf;
 		} else if (startswith(line, "SwapFree:") && memswlimit > 0 &&
-			   memswusage > 0 && opts && opts->swap_off == false) {
-			uint64_t swaptotal = memswlimit,
-				 swapusage = memusage > memswusage
-						 ? 0
-						 : memswusage - memusage,
-				 swapfree = swapusage < swaptotal
-						? swaptotal - swapusage
-						: 0;
-			snprintf(lbuf, 100, "SwapFree:       %8" PRIu64 " kB\n", swapfree);
+			   opts && opts->swap_off == false) {
+			uint64_t swfree = 0;
+			uint64_t swusage = 0;
+
+			swusage = memswusage - memusage;
+			swfree = swtotal - swusage;
+
+			snprintf(lbuf, 100, "SwapFree:       %8" PRIu64 " kB\n", swfree);
 			printme = lbuf;
 		} else if (startswith(line, "SwapFree:") && opts && opts->swap_off == true) {
 			snprintf(lbuf, 100, "SwapFree:       %8" PRIu64 " kB\n", (uint64_t)0);
@@ -1157,61 +1159,61 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 				  mstat.total_inactive_file) /
 				     1024);
 			printme = lbuf;
-		} else if (startswith(line, "Active(anon)")) {
+		} else if (startswith(line, "Active(anon):")) {
 			snprintf(lbuf, 100, "Active(anon):   %8" PRIu64 " kB\n",
 				 mstat.total_active_anon / 1024);
 			printme = lbuf;
-		} else if (startswith(line, "Inactive(anon)")) {
+		} else if (startswith(line, "Inactive(anon):")) {
 			snprintf(lbuf, 100, "Inactive(anon): %8" PRIu64 " kB\n",
 				 mstat.total_inactive_anon / 1024);
 			printme = lbuf;
-		} else if (startswith(line, "Active(file)")) {
+		} else if (startswith(line, "Active(file):")) {
 			snprintf(lbuf, 100, "Active(file):   %8" PRIu64 " kB\n",
 				 mstat.total_active_file / 1024);
 			printme = lbuf;
-		} else if (startswith(line, "Inactive(file)")) {
+		} else if (startswith(line, "Inactive(file):")) {
 			snprintf(lbuf, 100, "Inactive(file): %8" PRIu64 " kB\n",
 				 mstat.total_inactive_file / 1024);
 			printme = lbuf;
-		} else if (startswith(line, "Unevictable")) {
+		} else if (startswith(line, "Unevictable:")) {
 			snprintf(lbuf, 100, "Unevictable:    %8" PRIu64 " kB\n",
 				 mstat.total_unevictable / 1024);
 			printme = lbuf;
- 		} else if (startswith(line, "Dirty")) {
+ 		} else if (startswith(line, "Dirty:")) {
 			snprintf(lbuf, 100, "Dirty:          %8" PRIu64 " kB\n",
 				 mstat.total_dirty / 1024);
 			printme = lbuf;
- 		} else if (startswith(line, "Writeback")) {
+ 		} else if (startswith(line, "Writeback:")) {
 			snprintf(lbuf, 100, "Writeback:      %8" PRIu64 " kB\n",
 				 mstat.total_writeback / 1024);
 			printme = lbuf;
- 		} else if (startswith(line, "AnonPages")) {
+ 		} else if (startswith(line, "AnonPages:")) {
 			snprintf(lbuf, 100, "AnonPages:      %8" PRIu64 " kB\n",
 				 (mstat.total_active_anon +
 				  mstat.total_inactive_anon - mstat.total_shmem) /
 				     1024);
 			printme = lbuf;
- 		} else if (startswith(line, "Mapped")) {
+ 		} else if (startswith(line, "Mapped:")) {
 			snprintf(lbuf, 100, "Mapped:         %8" PRIu64 " kB\n",
 				 mstat.total_mapped_file / 1024);
 			printme = lbuf;
-		} else if (startswith(line, "SReclaimable")) {
+		} else if (startswith(line, "SReclaimable:")) {
 			snprintf(lbuf, 100, "SReclaimable:   %8" PRIu64 " kB\n", (uint64_t)0);
 			printme = lbuf;
-		} else if (startswith(line, "SUnreclaim")) {
+		} else if (startswith(line, "SUnreclaim:")) {
 			snprintf(lbuf, 100, "SUnreclaim:     %8" PRIu64 " kB\n", (uint64_t)0);
 			printme = lbuf;
 		} else if (startswith(line, "Shmem:")) {
 			snprintf(lbuf, 100, "Shmem:          %8" PRIu64 " kB\n",
 				 mstat.total_shmem / 1024);
 			printme = lbuf;
-		} else if (startswith(line, "ShmemHugePages")) {
+		} else if (startswith(line, "ShmemHugePages:")) {
 			snprintf(lbuf, 100, "ShmemHugePages: %8" PRIu64 " kB\n", (uint64_t)0);
 			printme = lbuf;
-		} else if (startswith(line, "ShmemPmdMapped")) {
+		} else if (startswith(line, "ShmemPmdMapped:")) {
 			snprintf(lbuf, 100, "ShmemPmdMapped: %8" PRIu64 " kB\n", (uint64_t)0);
  			printme = lbuf;
- 		} else if (startswith(line, "AnonHugePages")) {
+ 		} else if (startswith(line, "AnonHugePages:")) {
 			snprintf(lbuf, 100, "AnonHugePages:  %8" PRIu64 " kB\n",
 				 mstat.total_rss_huge / 1024);
 			printme = lbuf;
